@@ -1,6 +1,6 @@
-;; dos-w32.el --- Functions shared among MS-DOS and W32 (NT/95) platforms
+;;; dos-w32.el --- Functions shared among MS-DOS and W32 (NT/95) platforms  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1996, 2001-2019 Free Software Foundation, Inc.
+;; Copyright (C) 1996, 2001-2022 Free Software Foundation, Inc.
 
 ;; Maintainer: Geoff Voelker <voelker@cs.washington.edu>
 ;; Keywords: internal
@@ -154,13 +154,15 @@ when writing the file."
   ;; FIXME: Can't we use find-file-literally for the same purposes?
   (interactive "FFind file binary: ")
   (let ((coding-system-for-read 'no-conversion))  ;; FIXME: undecided-unix?
-    (find-file filename)))
+    (with-suppressed-warnings ((interactive-only find-file))
+      (find-file filename))))
 
 (defun find-file-text (filename)
   "Visit file FILENAME and treat it as a text file."
   (interactive "FFind file text: ")
   (let ((coding-system-for-read 'undecided-dos))
-    (find-file filename)))
+    (with-suppressed-warnings ((interactive-only find-file))
+      (find-file filename))))
 
 (defun w32-find-file-not-found-set-buffer-file-coding-system ()
   (with-current-buffer (current-buffer)
@@ -261,6 +263,8 @@ filesystem mounted on drive Z:, FILESYSTEM could be \"Z:\"."
   :group 'dos-fns
   :group 'w32)
 
+(defvar w32-quote-process-args)
+
 ;; Function to actually send data to the printer port.
 ;; Supports writing directly, and using various programs.
 (defun w32-direct-print-region-helper (printer
@@ -342,7 +346,7 @@ filesystem mounted on drive Z:, FILESYSTEM could be \"Z:\"."
 	       w32-direct-print-region-use-command-dot-com
 	       ;; file-attributes fails on LPT ports on Windows 9x but
 	       ;; not on NT, so handle both cases for safety.
-	       (eq (or (nth 7 (file-attributes printer)) 0) 0))
+	       (eq (or (file-attribute-size (file-attributes printer)) 0) 0))
 	  (write-region start end tempfile nil 0)
 	  (let ((w32-quote-process-args nil))
 	    (call-process "command.com" nil errbuf nil "/c"
