@@ -1,10 +1,9 @@
-;;; edt.el --- enhanced EDT keypad mode emulation for GNU Emacs
+;;; edt.el --- enhanced EDT keypad mode emulation for GNU Emacs  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1986, 1992-1995, 2000-2017 Free Software Foundation,
+;; Copyright (C) 1986, 1992-1995, 2000-2022 Free Software Foundation,
 ;; Inc.
 
 ;; Author: Kevin Gallagher <kevin.gal@verizon.net>
-;; Maintainer: Kevin Gallagher <kevin.gal@verizon.net>
 ;; Keywords: emulations
 
 ;; This file is part of GNU Emacs.
@@ -107,10 +106,7 @@
 ;;      customization file, edt-user.el, to do this.
 ;;      See Info node `edt' for more details.
 
-;;  3.  EDT Emulation now also works in XEmacs, including the
-;;      highlighting of selected text.
-
-;;  4.  If you access a workstation using an X Server, observe that
+;;  3.  If you access a workstation using an X Server, observe that
 ;;      the initialization file generated by edt-mapper.el will now
 ;;      contain the name of the X Server vendor.  This is a
 ;;      convenience for those who have access to their Unix account
@@ -121,7 +117,7 @@
 ;;      names.  Then, the correct initialization file for the
 ;;      particular X server in use is loaded correctly automatically.
 
-;;  5.  Also, edt-mapper.el is now capable of binding an ASCII key
+;;  4.  Also, edt-mapper.el is now capable of binding an ASCII key
 ;;      sequence, providing the ASCII key sequence prefix is already
 ;;      known by Emacs to be a prefix.  As a result of providing this
 ;;      support, some terminal/keyboard/window system configurations,
@@ -141,7 +137,7 @@
 ;;      your terminal OR a bug in the terminal emulation software you
 ;;      are using.)
 
-;;  6.  The edt-quit function (bound to GOLD-q by default) has been
+;;  5.  The edt-quit function (bound to GOLD-q by default) has been
 ;;      modified to warn the user when file-related buffer
 ;;      modifications exist.  It now cautions the user that those
 ;;      modifications will be lost if the user quits without saving
@@ -181,13 +177,6 @@
 (defvar edt-rect-start-point)
 (defvar edt-user-global-map)
 (defvar rect-start-point)
-(defvar time-string)
-(defvar zmacs-region-stays)
-
-;;;
-;;;  Version Information
-;;;
-(defconst edt-version "4.0" "EDT Emulation version number.")
 
 ;;;
 ;;;  User Configurable Variables
@@ -200,8 +189,7 @@ Emulation.  If set to nil (the default), the `page-delimiter' variable
 is set to \"\\f\" when edt-emulation-on is first invoked.  This
 setting replicates EDT's page delimiter behavior.  The original value
 is restored when edt-emulation-off is called."
-  :type 'boolean
-  :group 'edt)
+  :type 'boolean)
 
 (defcustom edt-use-EDT-control-key-bindings nil
   "Emacs MUST be restarted for a change in value to take effect!
@@ -209,8 +197,7 @@ Non-nil causes the control key bindings to be replaced with EDT
 bindings.  If set to nil (the default), EDT control key bindings are
 not used and the current Emacs control key bindings are retained for
 use within the EDT emulation."
-  :type 'boolean
-  :group 'edt)
+  :type 'boolean)
 
 (defcustom edt-word-entities '(?\t)
   "Specifies the list of EDT word entity characters.
@@ -234,22 +221,19 @@ representations, which you can also use:
 
 In EDT Emulation movement-by-word commands, each character in the list
 will be treated as if it were a separate word."
-  :type '(repeat integer)
-  :group 'edt)
+  :type '(repeat integer))
 
 (defcustom edt-top-scroll-margin 10
   "Scroll margin at the top of the screen.
 Interpreted as a percent of the current window size with a default
 setting of 10%.  If set to 0, top scroll margin is disabled."
-  :type 'integer
-  :group 'edt)
+  :type 'integer)
 
 (defcustom edt-bottom-scroll-margin 15
   "Scroll margin at the bottom of the screen.
 Interpreted as a percent of the current window size with a default
 setting of 15%.  If set to 0, bottom scroll margin is disabled."
-  :type 'integer
-  :group 'edt)
+  :type 'integer)
 
 ;;;
 ;;; Internal Variables
@@ -311,23 +295,15 @@ This means that an edt-user.el file was found in the user's `load-path'.")
 ;;;
 ;;;     o edt-emulation-on      o edt-load-keys
 ;;;
-(defconst edt-emacs-variant (if (featurep 'emacs) "gnu" "xemacs")
-  "Indicates Emacs variant:  GNU Emacs or XEmacs (aka Lucid Emacs).")
-
-(defconst edt-window-system (if (featurep 'emacs) window-system (console-type))
-  "Indicates window system (in GNU Emacs) or console type (in XEmacs).")
-
-(declare-function x-server-vendor "xfns.c" (&optional terminal))
-
-(defconst edt-xserver (when (eq edt-window-system 'x)
+(defconst edt-xserver (when (eq window-system 'x)
+                        (declare-function x-server-vendor "xfns.c"
+                                          (&optional terminal))
 			;; The Cygwin window manager has a `/' in its
 			;; name, which breaks the generated file name of
 			;; the custom key map file.  Replace `/' with a
 			;; `-' to work around that.
-			(if (featurep 'xemacs)
-			    (replace-in-string (x-server-vendor) "[ /]" "-")
-			  (replace-regexp-in-string "[ /]" "-"
-						    (x-server-vendor))))
+                        (replace-regexp-in-string "[ /]" "-"
+                                                  (x-server-vendor)))
   "Indicates X server vendor name, if applicable.")
 
 (defvar edt-keys-file nil
@@ -341,31 +317,31 @@ This means that an edt-user.el file was found in the user's `load-path'.")
 ;;;; EDT Emulation Commands
 ;;;;
 
-;;; Almost all of EDT's keypad mode commands have equivalent Emacs
-;;; function counterparts.  But many of these counterparts behave
-;;; somewhat differently in Emacs.
-;;;
-;;; So, the following Emacs functions emulate, where practical, the
-;;; exact behavior of the corresponding EDT keypad mode commands.  In
-;;; a few cases, the emulation is not exact, but it should be close
-;;; enough for most EDT die-hards.
-;;;
+;; Almost all of EDT's keypad mode commands have equivalent Emacs
+;; function counterparts.  But many of these counterparts behave
+;; somewhat differently in Emacs.
+;;
+;; So, the following Emacs functions emulate, where practical, the
+;; exact behavior of the corresponding EDT keypad mode commands.  In
+;; a few cases, the emulation is not exact, but it should be close
+;; enough for most EDT die-hards.
+;;
 
 ;;;
 ;;; PAGE
 ;;;
-;;; Emacs uses the regexp assigned to page-delimiter to determine what
-;;; marks a page break.  This is normally "^\f", which causes the
-;;; edt-page command to ignore form feeds not located at the beginning
-;;; of a line.  To emulate the EDT PAGE command exactly,
-;;; page-delimiter is set to "\f" when EDT emulation is turned on, and
-;;; restored to its original value when EDT emulation is turned off.
-;;; But this can be overridden if the EDT definition is not desired by
-;;; placing
-;;;
-;;;         (setq edt-keep-current-page-delimiter t)
-;;;
-;;; in your init file.
+;; Emacs uses the regexp assigned to page-delimiter to determine what
+;; marks a page break.  This is normally "^\f", which causes the
+;; edt-page command to ignore form feeds not located at the beginning
+;; of a line.  To emulate the EDT PAGE command exactly,
+;; page-delimiter is set to "\f" when EDT emulation is turned on, and
+;; restored to its original value when EDT emulation is turned off.
+;; But this can be overridden if the EDT definition is not desired by
+;; placing
+;;
+;;         (setq edt-keep-current-page-delimiter t)
+;;
+;; in your init file.
 
 (defun edt-page-forward (num)
   "Move forward to just after next page delimiter.
@@ -389,8 +365,7 @@ Argument NUM is the number of page delimiters to move."
       (error "Beginning of buffer")
 	(progn
 	  (backward-page num)
-      (edt-line-to-top-of-window)
-	  (if (featurep 'xemacs) (setq zmacs-region-stays t)))))
+      (edt-line-to-top-of-window))))
 
 (defun edt-page (num)
   "Move in current direction to next page delimiter.
@@ -403,12 +378,12 @@ Argument NUM is the number of page delimiters to move."
 ;;;
 ;;; SECT
 ;;;
-;;; EDT defaults a section size to be 16 lines of its one and only
-;;; 24-line window.  That's two-thirds of the window at a time.  The
-;;; EDT SECT commands moves the cursor, not the window.
-;;;
-;;; This emulation of EDT's SECT moves the cursor approximately
-;;; two-thirds of the current window at a time.
+;; EDT defaults a section size to be 16 lines of its one and only
+;; 24-line window.  That's two-thirds of the window at a time.  The
+;; EDT SECT commands moves the cursor, not the window.
+;;
+;; This emulation of EDT's SECT moves the cursor approximately
+;; two-thirds of the current window at a time.
 
 (defun edt-sect-forward (num)
   "Move cursor forward two-thirds of a window's number of lines.
@@ -436,8 +411,8 @@ Argument NUM is the number of sections to move."
 ;;;
 ;;; BEGINNING OF LINE
 ;;;
-;;; EDT's beginning-of-line command is not affected by current
-;;; direction, for some unknown reason.
+;; EDT's beginning-of-line command is not affected by current
+;; direction, for some unknown reason.
 
 (defun edt-beginning-of-line (num)
   "Move backward to next beginning of line mark.
@@ -450,8 +425,7 @@ Argument NUM is the number of BOL marks to move."
       (progn
 	(setq num (1- num))
 	(forward-line (* -1 num))))
-    (edt-top-check beg num))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+    (edt-top-check beg num)))
 
 
 ;;;
@@ -466,8 +440,7 @@ Argument NUM is the number of EOL marks to move."
   (let ((beg (edt-current-line)))
     (forward-char)
     (end-of-line num)
-    (edt-bottom-check beg num))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+    (edt-bottom-check beg num)))
 
 
 (defun edt-end-of-line-backward (num)
@@ -477,8 +450,7 @@ Argument NUM is the number of EOL marks to move."
   (edt-check-prefix num)
   (let ((beg (edt-current-line)))
     (end-of-line (1- num))
-    (edt-top-check beg num))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+    (edt-top-check beg num)))
 
 
 (defun edt-end-of-line (num)
@@ -492,13 +464,13 @@ Argument NUM is the number of EOL marks to move."
 ;;;
 ;;; WORD
 ;;;
-;;; This one is a tad messy.  To emulate EDT's behavior everywhere in
-;;; the file (beginning of file, end of file, beginning of line, end
-;;; of line, etc.) it takes a bit of special handling.
-;;;
-;;; The variable edt-word-entities contains a list of characters which
-;;; are to be viewed as distinct words wherever they appear in the
-;;; buffer.  This emulates the EDT line mode command SET ENTITY WORD.
+;; This one is a tad messy.  To emulate EDT's behavior everywhere in
+;; the file (beginning of file, end of file, beginning of line, end
+;; of line, etc.) it takes a bit of special handling.
+;;
+;; The variable edt-word-entities contains a list of characters which
+;; are to be viewed as distinct words wherever they appear in the
+;; buffer.  This emulates the EDT line mode command SET ENTITY WORD.
 
 
 (defun edt-one-word-forward ()
@@ -522,8 +494,7 @@ Argument NUM is the number of EOL marks to move."
 	      (not (eobp))
 	      (eq ?\  (char-syntax (following-char)))
 	      (not (memq (following-char) edt-word-entities)))
-	(forward-char))))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+	(forward-char)))))
 
 (defun edt-one-word-backward ()
   "Move backward to first character of previous word."
@@ -546,8 +517,7 @@ Argument NUM is the number of EOL marks to move."
 		  (not (bobp))
 		  (not (eq ?\  (char-syntax (preceding-char))))
 		  (not (memq (preceding-char) edt-word-entities)))
-	    (backward-char)))))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+	    (backward-char))))))
 
 (defun edt-word-forward (num)
   "Move forward to first character of next word.
@@ -586,15 +556,14 @@ Argument NUM is the number of characters to move."
   (edt-check-prefix num)
   (if (equal edt-direction-string edt-forward-string)
       (forward-char num)
-    (backward-char num))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+    (backward-char num)))
 
 ;;;
 ;;; LINE
 ;;;
-;;; When direction is set to BACKUP, LINE behaves just like BEGINNING
-;;; OF LINE in EDT.  So edt-line-backward is not really needed as a
-;;; separate function.
+;; When direction is set to BACKUP, LINE behaves just like BEGINNING
+;; OF LINE in EDT.  So edt-line-backward is not really needed as a
+;; separate function.
 
 (defun edt-line-backward (num)
   "Move backward to next beginning of line mark.
@@ -609,8 +578,7 @@ Argument NUM is the number of BOL marks to move."
   (edt-check-prefix num)
   (let ((beg (edt-current-line)))
     (forward-line num)
-    (edt-bottom-check beg num))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+    (edt-bottom-check beg num)))
 
 (defun edt-line (num)
   "Move in current direction to next beginning of line mark.
@@ -632,8 +600,7 @@ Argument NUM is the number of lines to move."
   (let ((beg (edt-current-line)))
     ;; We're deliberately using next-line instead of forward-line.
     (with-no-warnings (next-line num))
-    (edt-bottom-check beg num))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+    (edt-bottom-check beg num)))
 
 (defun edt-previous-line (num)
   "Move cursor up one line.
@@ -643,8 +610,7 @@ Argument NUM is the number of lines to move."
   (let ((beg (edt-current-line)))
     ;; We're deliberately using previous-line instead of forward-line.
     (with-no-warnings (previous-line num))
-    (edt-top-check beg num))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+    (edt-top-check beg num)))
 
 
 ;;;
@@ -654,8 +620,7 @@ Argument NUM is the number of lines to move."
 (defun edt-top ()
   "Move cursor to the beginning of buffer."
   (interactive)
-  (goto-char (point-min))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+  (goto-char (point-min)))
 
 ;;;
 ;;; BOTTOM
@@ -669,8 +634,7 @@ Argument NUM is the number of lines to move."
 
 (defmacro edt-with-position (&rest body)
   "Execute BODY with some position-related variables bound."
-  `(let* ((left nil)
-          (beg (edt-current-line))
+  `(let* ((beg (edt-current-line))
           (height (window-height))
           (top-percent
            (if (zerop edt-top-scroll-margin) 10 edt-top-scroll-margin))
@@ -684,6 +648,7 @@ Argument NUM is the number of lines to move."
           (far (save-excursion
                  (goto-char bottom)
                  (point-at-bol (1- height)))))
+     (ignore top far)
      ,@body))
 
 ;;;
@@ -701,11 +666,11 @@ Optional argument FIND is t is this function is called from `edt-find'."
      (search-backward edt-find-last-text)
      (edt-set-match)
      (if (> (point) far)
-         (if (zerop (setq left (save-excursion (forward-line height))))
-             (recenter top-margin)
-           (recenter (- left bottom-up-margin)))
-       (and (> (point) bottom) (recenter bottom-margin)))))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+         (let ((left (save-excursion (forward-line height))))
+           (recenter (if (zerop left)
+                         top-margin
+                       (- left bottom-up-margin))))
+       (and (> (point) bottom) (recenter bottom-margin))))))
 
 (defun edt-find-backward (&optional find)
   "Find first occurrence of a string in the backward direction and save it.
@@ -716,13 +681,12 @@ Optional argument FIND is t if this function is called from `edt-find'."
   (edt-with-position
    (if (search-backward edt-find-last-text)
        (edt-set-match))
-   (and (< (point) top) (recenter (min beg top-margin))))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+   (and (< (point) top) (recenter (min beg top-margin)))))
 
 (defun edt-find ()
   "Find first occurrence of string in current direction and save it."
   (interactive)
-  (set 'edt-find-last-text (read-string "Search: "))
+  (setq edt-find-last-text (read-string "Search: "))
   (if (equal edt-direction-string edt-forward-string)
       (edt-find-forward t)
       (edt-find-backward t)))
@@ -742,13 +706,12 @@ Optional argument FIND is t if this function is called from `edt-find'."
          (search-backward edt-find-last-text)
          (edt-set-match)
          (if (> (point) far)
-             (if (zerop (setq left (save-excursion (forward-line height))))
-                 (recenter top-margin)
-               (recenter (- left bottom-up-margin)))
+             (let ((left (save-excursion (forward-line height))))
+               (recenter (if (zerop left) top-margin
+                           (- left bottom-up-margin))))
            (and (> (point) bottom) (recenter bottom-margin))))
      (backward-char 1)
-     (error "Search failed: \"%s\"" edt-find-last-text)))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+     (error "Search failed: \"%s\"" edt-find-last-text))))
 
 (defun edt-find-next-backward ()
   "Find next occurrence of a string in backward direction."
@@ -757,8 +720,7 @@ Optional argument FIND is t if this function is called from `edt-find'."
    (if (not (search-backward edt-find-last-text nil t))
        (error "Search failed: \"%s\"" edt-find-last-text)
      (edt-set-match)
-     (and (< (point) top) (recenter (min beg top-margin)))))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+     (and (< (point) top) (recenter (min beg top-margin))))))
 
 (defun edt-find-next ()
   "Find next occurrence of a string in current direction."
@@ -821,7 +783,7 @@ Argument NUM is the number of lines to delete."
 In select mode, selected text is highlighted."
   (if arg
       (progn
-	(set (make-local-variable 'edt-select-mode) 'edt-select-mode-current)
+        (setq-local edt-select-mode 'edt-select-mode-current)
 	(setq rect-start-point (window-point)))
     (progn
       (kill-local-variable 'edt-select-mode)))
@@ -835,9 +797,7 @@ In select mode, selected text is highlighted."
 (defun edt-reset ()
   "Cancel text selection."
   (interactive)
-  (if (featurep 'emacs)
-      (deactivate-mark)
-    (zmacs-deactivate-region)))
+  (deactivate-mark))
 
 ;;;
 ;;; CUT
@@ -1051,8 +1011,7 @@ Also, execute command specified if in Minibuffer."
   (force-mode-line-update)
   (if (string-equal " *Minibuf"
                     (substring (buffer-name) 0 (min (length (buffer-name)) 9)))
-      (exit-minibuffer))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+      (exit-minibuffer)))
 
 
 ;;;
@@ -1067,8 +1026,7 @@ Also, execute command specified if in Minibuffer."
   (force-mode-line-update)
   (if (string-equal " *Minibuf"
                     (substring (buffer-name) 0 (min (length (buffer-name)) 9)))
-      (exit-minibuffer))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+      (exit-minibuffer)))
 
 
 ;;;
@@ -1118,14 +1076,11 @@ Argument NUM is the numbers of consecutive characters to change."
 The current key definition is saved in `edt-last-replaced-key-definition'.
 Use `edt-restore-key' to restore last replaced key definition."
   (interactive)
-  (if (featurep 'xemacs) (setq zmacs-region-stays t))
   (let (edt-function
 	edt-key-definition)
     (setq edt-key-definition
 	  (read-key-sequence "Press the key to be defined: "))
-    (if (if (featurep 'emacs)
-	    (string-equal "\C-m" edt-key-definition)
-	  (string-equal "\C-m" (events-to-keys edt-key-definition)))
+    (if (string-equal "\C-m" edt-key-definition)
         (message "Key not defined")
       (progn
 	(setq edt-function (read-command "Enter command name: "))
@@ -1203,8 +1158,6 @@ Argument LINES is the number of lines the cursor moved toward the bottom."
 		 ;; subtract 1 from height because it includes mode line
 		 (difference (- height margin 1)))
     (cond ((> beg difference) (recenter beg))
-		  ((and (featurep 'xemacs) (> (+ beg lines 1) difference))
-		   (recenter (- margin)))
 		  ((> (+ beg lines) difference) (recenter (- margin))))))
 
 (defun edt-current-line nil
@@ -1245,9 +1198,9 @@ Argument BOTTOM is the bottom margin in number of lines or percent of window."
 ;;;;
 
 ;;;
-;;; Several enhancements and additions to EDT keypad mode commands are
-;;; provided here.  Some of these have been motivated by similar
-;;; TPU/EVE and EVE-Plus commands.  Others are new.
+;; Several enhancements and additions to EDT keypad mode commands are
+;; provided here.  Some of these have been motivated by similar
+;; TPU/EVE and EVE-Plus commands.  Others are new.
 
 ;;;
 ;;; CHANGE DIRECTION
@@ -1287,11 +1240,9 @@ Argument NUM is the positive number of sentences to move."
      (forward-word 1)
      (backward-sentence))
    (if (> (point) far)
-       (if (zerop (setq left (save-excursion (forward-line height))))
-           (recenter top-margin)
-         (recenter (- left bottom-up-margin)))
-     (and (> (point) bottom) (recenter bottom-margin))))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+       (let ((left (save-excursion (forward-line height))))
+         (recenter (if (zerop left) top-margin (- left bottom-up-margin))))
+     (and (> (point) bottom) (recenter bottom-margin)))))
 
 (defun edt-sentence-backward (num)
   "Move backward to next sentence beginning.
@@ -1302,8 +1253,7 @@ Argument NUM is the positive number of sentences to move."
    (if (eobp)
        (error "End of buffer")
      (backward-sentence num))
-   (and (< (point) top) (recenter (min beg top-margin))))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+   (and (< (point) top) (recenter (min beg top-margin)))))
 
 (defun edt-sentence (num)
   "Move in current direction to next sentence.
@@ -1330,11 +1280,9 @@ Argument NUM is the positive number of paragraphs to move."
          (forward-line 1))
      (setq num (1- num)))
    (if (> (point) far)
-       (if (zerop (setq left (save-excursion (forward-line height))))
-           (recenter top-margin)
-         (recenter (- left bottom-up-margin)))
-     (and (> (point) bottom) (recenter bottom-margin))))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+       (let ((left (save-excursion (forward-line height))))
+         (recenter (if (zerop left) top-margin (- left bottom-up-margin))))
+     (and (> (point) bottom) (recenter bottom-margin)))))
 
 (defun edt-paragraph-backward (num)
   "Move backward to beginning of paragraph.
@@ -1345,8 +1293,7 @@ Argument NUM is the positive number of paragraphs to move."
    (while (> num 0)
      (start-of-paragraph-text)
      (setq num (1- num)))
-   (and (< (point) top) (recenter (min beg top-margin))))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+   (and (< (point) top) (recenter (min beg top-margin)))))
 
 (defun edt-paragraph (num)
   "Move in current direction to next paragraph.
@@ -1364,24 +1311,18 @@ Argument NUM is the positive number of paragraphs to move."
   "Restore last replaced key definition.
 Definition is stored in `edt-last-replaced-key-definition'."
   (interactive)
-  (if (featurep 'xemacs) (setq zmacs-region-stays t))
   (if edt-last-replaced-key-definition
       (progn
         (let (edt-key-definition)
-          (set 'edt-key-definition
-               (read-key-sequence "Press the key to be restored: "))
-	  (if (if (featurep 'emacs)
-		  (string-equal "\C-m" edt-key-definition)
-		(string-equal "\C-m" (events-to-keys edt-key-definition)))
+          (setq edt-key-definition
+                (read-key-sequence "Press the key to be restored: "))
+          (if (string-equal "\C-m" edt-key-definition)
               (message "Key not restored")
 	    (progn
 	      (define-key (current-global-map)
 		edt-key-definition edt-last-replaced-key-definition)
-	      (if (featurep 'emacs)
-		  (message "Key definition for %s has been restored."
-			   edt-key-definition)
-		(message "Key definition for %s has been restored."
-			 (events-to-keys edt-key-definition)))))))
+              (message "Key definition for %s has been restored."
+                       edt-key-definition)))))
     (error "No replaced key definition to restore!")))
 
 ;;;
@@ -1393,8 +1334,7 @@ Definition is stored in `edt-last-replaced-key-definition'."
   (interactive)
   (let ((start-column (current-column)))
     (move-to-window-line 0)
-    (move-to-column start-column))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+    (move-to-column start-column)))
 
 ;;;
 ;;; WINDOW BOTTOM
@@ -1405,8 +1345,7 @@ Definition is stored in `edt-last-replaced-key-definition'."
   (interactive)
   (let ((start-column (current-column)))
     (move-to-window-line (- (window-height) 2))
-    (move-to-column start-column))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+    (move-to-column start-column)))
 
 ;;;
 ;;; SCROLL WINDOW LINE
@@ -1415,14 +1354,12 @@ Definition is stored in `edt-last-replaced-key-definition'."
 (defun edt-scroll-window-forward-line ()
   "Move window forward one line leaving cursor at position in window."
   (interactive)
-  (scroll-up 1)
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+  (scroll-up 1))
 
 (defun edt-scroll-window-backward-line ()
   "Move window backward one line leaving cursor at position in window."
   (interactive)
-  (scroll-down 1)
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+  (scroll-down 1))
 
 (defun edt-scroll-line ()
   "Move window one line in current direction."
@@ -1434,8 +1371,8 @@ Definition is stored in `edt-last-replaced-key-definition'."
 ;;;
 ;;; SCROLL WINDOW
 ;;;
-;;; Scroll a window (less one line) at a time.  Leave cursor in center of
-;;; window.
+;; Scroll a window (less one line) at a time.  Leave cursor in center of
+;; window.
 
 (defun edt-scroll-window-forward (num)
   "Scroll forward one window in buffer, less one line.
@@ -1468,8 +1405,7 @@ Argument NUM is the positive number of windows to move."
 (defun edt-line-to-bottom-of-window ()
   "Move the current line to the bottom of the window."
   (interactive)
-  (recenter -1)
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+  (recenter -1))
 
 ;;;
 ;;; LINE TO TOP OF WINDOW
@@ -1478,8 +1414,7 @@ Argument NUM is the positive number of windows to move."
 (defun edt-line-to-top-of-window ()
   "Move the current line to the top of the window."
   (interactive)
-  (recenter 0)
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+  (recenter 0))
 
 ;;;
 ;;; LINE TO MIDDLE OF WINDOW
@@ -1488,8 +1423,7 @@ Argument NUM is the positive number of windows to move."
 (defun edt-line-to-middle-of-window ()
   "Move window so line with cursor is in the middle of the window."
   (interactive)
-  (recenter '(4))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+  (recenter '(4)))
 
 ;;;
 ;;; GOTO PERCENTAGE
@@ -1501,8 +1435,7 @@ Argument NUM is the percentage into the buffer to move."
   (interactive "NGoto-percentage: ")
   (if (or (> num 100) (< num 0))
       (error "Percentage %d out of range 0 < percent < 100" num)
-    (goto-char (/ (* (point-max) num) 100)))
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+    (goto-char (/ (* (point-max) num) 100))))
 
 ;;;
 ;;; FILL REGION
@@ -1674,7 +1607,6 @@ Argument NUM is the number of times to duplicate the line."
 (defun edt-display-the-time ()
   "Display the current time."
   (interactive)
-  (if (featurep 'xemacs) (setq zmacs-region-stays t))
   (message "%s" (current-time-string)))
 
 ;;;
@@ -1700,14 +1632,12 @@ Argument NUM is the number of times to duplicate the line."
     (progn
       (end-kbd-macro nil)
       (let (edt-key-definition)
-	(set 'edt-key-definition
-	     (read-key-sequence "Enter key for binding: "))
-	(if (if (featurep 'emacs)
-		(string-equal "\C-m" edt-key-definition)
-	      (string-equal "\C-m" (events-to-keys edt-key-definition)))
+        (setq edt-key-definition
+              (read-key-sequence "Enter key for binding: "))
+	(if (string-equal "\C-m" edt-key-definition)
 	    (message "Key sequence not remembered")
 	  (progn
-	    (set 'edt-learn-macro-count (+ edt-learn-macro-count 1))
+            (setq edt-learn-macro-count (+ edt-learn-macro-count 1))
 	    (setq edt-last-replaced-key-definition
 		  (lookup-key (current-global-map)
 			      edt-key-definition))
@@ -1754,8 +1684,7 @@ Warn user that modifications will be lost."
   "Split current window and place cursor in the new window."
   (interactive)
   (split-window)
-  (other-window 1)
-  (if (featurep 'xemacs) (setq zmacs-region-stays t)))
+  (other-window 1))
 
 ;;;
 ;;; COPY RECTANGLE
@@ -1936,9 +1865,8 @@ Optional argument NOT-YES changes the default to negative."
 (defun edt-load-keys (file)
   "Load the LK-201 key mapping FILE generated by edt-mapper.el.
 If FILE is nil, which is the normal case, try to load a default file.
-The default file names are based upon the window system, terminal
-type, and version of Emacs in use: GNU Emacs or XEmacs (aka Lucid
-Emacs).  If a default file does not exist, ask user if one should be
+The default file names are based upon the window system and terminal
+type.  If a default file does not exist, ask user if one should be
 created."
   (interactive "fKey definition file: ")
   (cond (file
@@ -1949,11 +1877,11 @@ created."
 	 (setq file
 	       (expand-file-name
 		(concat
-		 "~/.edt-" edt-emacs-variant
+                 "~/.edt-gnu"
 		 (if edt-term (concat "-" edt-term))
 		 (if edt-xserver (concat "-" edt-xserver))
-		 (if edt-window-system
-		     (concat "-" (upcase (symbol-name edt-window-system))))
+                 (if window-system
+                     (concat "-" (upcase (symbol-name window-system))))
 		 "-keys")))))
   (cond ((file-readable-p file)
 	 (load-file file))
@@ -2003,15 +1931,14 @@ created."
   "Turn on EDT Emulation."
   (interactive)
   ;; If using pc window system (MS-DOS), set terminal type to pc.
-  ;; If not a window system (GNU) or a tty window system (XEmacs),
-  ;; get terminal type.
-  (if (eq edt-window-system 'pc)
+  ;; If not a window system, get terminal type.
+  (if (eq window-system 'pc)
       (setq edt-term "pc")
-    (if (or (not edt-window-system) (eq edt-window-system 'tty))
+    (if (not window-system)
 	(setq edt-term (getenv "TERM"))))
   ;; Look for a terminal configuration file for this terminal type.
   ;; Otherwise, load the user's custom configuration file.
-  (if (or (not edt-window-system) (memq edt-window-system '(pc tty)))
+  (if (or (not window-system) (memq window-system '(pc tty)))
       (progn
 	;; Load terminal-specific configuration file, if it exists for this
 	;; terminal type.  Note: All DEC VT series terminals are supported
@@ -2038,27 +1965,14 @@ created."
 	    (setq edt-term term))))
     (edt-load-keys nil))
   ;; Make highlighting of selected text work properly for EDT commands.
-  (if (featurep 'emacs)
-      (progn
-	(setq edt-orig-transient-mark-mode
-              (default-value 'transient-mark-mode))
-	(add-hook 'activate-mark-hook
-		  (function
-		   (lambda ()
-		     (edt-select-mode t))))
-	(add-hook 'deactivate-mark-hook
-		  (function
-		   (lambda ()
-		     (edt-select-mode nil)))))
-    (progn
-      (add-hook 'zmacs-activate-region-hook
-		(function
-		 (lambda ()
-		   (edt-select-mode t))))
-      (add-hook 'zmacs-deactivate-region-hook
-		(function
-		 (lambda ()
-		   (edt-select-mode nil))))))
+  (setq edt-orig-transient-mark-mode
+        (default-value 'transient-mark-mode))
+  (add-hook 'activate-mark-hook
+            (lambda ()
+              (edt-select-mode t)))
+  (add-hook 'deactivate-mark-hook
+            (lambda ()
+              (edt-select-mode nil)))
   ;;  Load user's EDT custom key bindings file, if it exists.
   ;;  Otherwise, use the default bindings.
   (if (load "edt-user" t t)
@@ -2075,8 +1989,7 @@ created."
   (setq edt-select-mode-current nil)
   (edt-reset)
   (force-mode-line-update t)
-  (if (featurep 'emacs)
-      (setq-default transient-mark-mode edt-orig-transient-mark-mode))
+  (setq-default transient-mark-mode edt-orig-transient-mark-mode)
   (message "Original key bindings restored; EDT Emulation disabled"))
 
 (defun edt-default-menu-bar-update-buffers ()
@@ -2104,9 +2017,7 @@ Optional argument USER-SETUP non-nil means  called from function
   ;; disturbing the original bindings in global-map.
   (fset 'edt-default-ESC-prefix (copy-keymap 'ESC-prefix))
   (setq edt-default-global-map (copy-keymap (current-global-map)))
-  (if (featurep 'emacs)
-      (define-key edt-default-global-map "\e" 'edt-default-ESC-prefix)
-    (define-key edt-default-global-map [escape] 'edt-default-ESC-prefix))
+  (define-key edt-default-global-map "\e" 'edt-default-ESC-prefix)
   (define-prefix-command 'edt-default-gold-map)
   (edt-setup-default-bindings)
   ;; If terminal has additional function keys, the terminal-specific
@@ -2133,7 +2044,7 @@ Optional argument USER-SETUP non-nil means  called from function
       (fset 'edt-emulation-on (symbol-function 'edt-select-default-global-map))
       (edt-select-default-global-map)))
   ;; Keep the menu bar Buffers menu up-to-date in edt-default-global-map.
-  (add-hook 'menu-bar-update-hook 'edt-default-menu-bar-update-buffers))
+  (add-hook 'menu-bar-update-hook #'edt-default-menu-bar-update-buffers))
 
 (defun edt-user-emulation-setup ()
   "Setup user custom emulation of DEC's EDT editor."
@@ -2142,9 +2053,7 @@ Optional argument USER-SETUP non-nil means  called from function
   ;; Setup user EDT global map by copying default EDT global map bindings.
   (fset 'edt-user-ESC-prefix (copy-keymap 'edt-default-ESC-prefix))
   (setq edt-user-global-map (copy-keymap edt-default-global-map))
-  (if (featurep 'emacs)
-      (define-key edt-user-global-map "\e" 'edt-user-ESC-prefix)
-    (define-key edt-user-global-map [escape] 'edt-user-ESC-prefix))
+  (define-key edt-user-global-map "\e" 'edt-user-ESC-prefix)
   ;; If terminal has additional function keys, the user's initialization
   ;; file can assign bindings to them via the optional
   ;; function edt-setup-extra-default-bindings.
@@ -2156,13 +2065,12 @@ Optional argument USER-SETUP non-nil means  called from function
       (edt-setup-user-bindings))
   (edt-select-user-global-map)
   ;; Keep the menu bar Buffers menu up-to-date in edt-user-global-map.
-  (add-hook 'menu-bar-update-hook 'edt-user-menu-bar-update-buffers))
+  (add-hook 'menu-bar-update-hook #'edt-user-menu-bar-update-buffers))
 
 (defun edt-select-default-global-map()
   "Select default EDT emulation key bindings."
   (interactive)
-  (if (featurep 'emacs)
-    (transient-mark-mode 1))
+  (transient-mark-mode 1)
   (use-global-map edt-default-global-map)
   (if (not edt-keep-current-page-delimiter)
       (progn
@@ -2179,8 +2087,7 @@ Optional argument USER-SETUP non-nil means  called from function
   (interactive)
   (if edt-user-map-configured
       (progn
-	(if (featurep 'emacs)
-	    (transient-mark-mode 1))
+        (transient-mark-mode 1)
         (use-global-map edt-user-global-map)
         (if (not edt-keep-current-page-delimiter)
             (progn
@@ -2247,8 +2154,7 @@ Argument KEY is the name of a key.  It can be a standard key or a function key.
 Argument BINDING is the Emacs function to be bound to <KEY>."
   (define-key edt-user-global-map key binding))
 
-;;  For backward compatibility to existing edt-user.el files.
-(fset 'edt-bind-standard-key (symbol-function 'edt-bind-key))
+(define-obsolete-function-alias 'edt-bind-standard-key #'edt-bind-key "28.1")
 
 (defun edt-bind-gold-key (key gold-binding)
   "Binds <GOLD> standard key sequences to custom bindings in the EDT Emulator.
@@ -2577,7 +2483,7 @@ G-C-\\: Split Window                     |  FNDNXT  |   Yank   |   CUT    |
                   (and b
                        (with-current-buffer b
                          (set-buffer-modified-p t)))
-                  (fset 'help-print-return-message 'ignore)
+                  (fset 'help-print-return-message #'ignore)
                   (call-interactively fun)
                   (and (get-buffer name)
                        (get-buffer-window (get-buffer name))
@@ -2624,6 +2530,9 @@ G-C-\\: Split Window                     |  FNDNXT  |   Yank   |   CUT    |
   (set-frame-width nil 132)
   (message "Terminal width 132"))
 
+(defconst edt-version "4.0" "EDT Emulation version number.")
+(make-obsolete-variable 'edt-version 'emacs-version "28.1")
+
 (provide 'edt)
 
 ;;; edt.el ends here
