@@ -1,6 +1,6 @@
-;;; gnus-html.el --- Render HTML in a buffer.
+;;; gnus-html.el --- Render HTML in a buffer.  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 2010-2019 Free Software Foundation, Inc.
+;; Copyright (C) 2010-2022 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
 ;; Keywords: html, web
@@ -28,8 +28,6 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
-
 (require 'gnus-art)
 (eval-when-compile (require 'mm-decode))
 
@@ -86,7 +84,7 @@ fit these criteria."
     (define-key map "i" 'gnus-html-browse-image)
     (define-key map "\r" 'gnus-html-browse-url)
     (define-key map "u" 'gnus-article-copy-string)
-    (define-key map [tab] 'widget-forward)
+    (define-key map [tab] 'button-forward)
     map))
 
 (defun gnus-html-encode-url (url)
@@ -99,11 +97,7 @@ fit these criteria."
          (not (file-exists-p (url-cache-create-filename url))))
         (t (let ((cache-time (url-is-cached url)))
              (if cache-time
-                 (time-less-p
-                  (time-add
-                   cache-time
-                   ttl)
-                  (current-time))
+                 (time-less-p (time-add cache-time ttl) nil)
                t)))))
 
 ;;;###autoload
@@ -157,8 +151,8 @@ fit these criteria."
 
 (defun gnus-html-wash-images ()
   "Run through current buffer and replace img tags by images."
-  (let (tag parameters string start end images url alt-text
-	    inhibit-images blocked-images)
+  (let ( parameters start end ;; tag string images
+	 inhibit-images blocked-images)
     (if (buffer-live-p gnus-summary-buffer)
 	(with-current-buffer gnus-summary-buffer
 	  (setq inhibit-images gnus-inhibit-images
@@ -175,75 +169,67 @@ fit these criteria."
 	(delete-region (match-beginning 0) (match-end 0)))
       (setq end (point))
       (when (string-match "src=\"\\([^\"]+\\)" parameters)
-	(gnus-message 8 "gnus-html-wash-tags: fetching image URL %s" url)
-	(setq url (gnus-html-encode-url (match-string 1 parameters))
-	      alt-text (when (string-match "\\(alt\\|title\\)=\"\\([^\"]+\\)"
-					   parameters)
-			 (xml-substitute-special (match-string 2 parameters))))
-	(add-text-properties
-	 start end
-	 (list 'image-url url
-	       'image-displayer `(lambda (url start end)
-				   (gnus-html-display-image url start end
-							    ,alt-text))
-	       'gnus-image (list url start end alt-text)))
-	(widget-convert-button
-	 'url-link start (point)
-	 :help-echo alt-text
-	 :keymap gnus-html-image-map
-	 url)
-	(if (string-match "\\`cid:" url)
-	    ;; URLs with cid: have their content stashed in other
-	    ;; parts of the MIME structure, so just insert them
-	    ;; immediately.
-	    (let* ((handle (mm-get-content-id (substring url (match-end 0))))
-		   (image (when (and handle
-				     (not inhibit-images))
-			    (gnus-create-image
-			     (mm-with-part handle (buffer-string))
-			     nil t))))
-	      (if image
-		  (gnus-add-image
-		   'cid
-		   (gnus-put-image
-		    (gnus-rescale-image
-		     image (gnus-html-maximum-image-size))
-		    (gnus-string-or (prog1
-					(buffer-substring start end)
-				      (delete-region start end))
-				    "*")
-		    'cid))
-		(widget-convert-button
-		 'link start end
-		 :action 'gnus-html-insert-image
-		 :help-echo url
-		 :keymap gnus-html-image-map
-		 :button-keymap gnus-html-image-map)))
-	  ;; Normal, external URL.
-	  (if (or inhibit-images
-		  (gnus-html-image-url-blocked-p url blocked-images))
-	      (widget-convert-button
-	       'link start end
-	       :action 'gnus-html-insert-image
-	       :help-echo url
-	       :keymap gnus-html-image-map
-	       :button-keymap gnus-html-image-map)
-	    ;; Non-blocked url
-	    (let ((width
-		   (when (string-match "width=\"?\\([0-9]+\\)" parameters)
-		     (string-to-number (match-string 1 parameters))))
-		  (height
-		   (when (string-match "height=\"?\\([0-9]+\\)" parameters)
-		     (string-to-number (match-string 1 parameters)))))
-	      ;; Don't fetch images that are really small.  They're
-	      ;; probably tracking pictures.
-	      (when (and (or (null height)
-			     (> height 4))
-			 (or (null width)
-			     (> width 4)))
-		(gnus-html-display-image url start end alt-text)))))))))
+	(let ((url (gnus-html-encode-url (match-string 1 parameters)))
+	      (alt-text (when (string-match "\\(alt\\|title\\)=\"\\([^\"]+\\)"
+					    parameters)
+			  (xml-substitute-special (match-string 2 parameters)))))
+	  (gnus-message 8 "gnus-html-wash-tags: fetching image URL %s" url)
+	  (add-text-properties
+	   start end
+	   (list 'image-url url
+		 'image-displayer (lambda (url start end)
+				    (gnus-html-display-image url start end
+							     alt-text))
+		 'help-echo alt-text
+		 'button t
+		 'keymap gnus-html-image-map
+		 'gnus-image (list url start end alt-text)))
+	  (if (string-match "\\`cid:" url)
+	      ;; URLs with cid: have their content stashed in other
+	      ;; parts of the MIME structure, so just insert them
+	      ;; immediately.
+	      (let* ((handle (mm-get-content-id (substring url (match-end 0))))
+		     (image (when (and handle
+				       (not inhibit-images))
+			      (gnus-create-image
+			       (mm-with-part handle (buffer-string))
+			       nil t))))
+		(if image
+		    (gnus-add-image
+		     'cid
+		     (gnus-put-image
+		      (gnus-rescale-image
+		       image (gnus-html-maximum-image-size))
+		      (gnus-string-or (prog1
+					  (buffer-substring start end)
+					(delete-region start end))
+				      "*")
+		      'cid))
+		  (make-text-button start end
+				    'help-echo url
+				    'keymap gnus-html-image-map)))
+	    ;; Normal, external URL.
+	    (if (or inhibit-images
+		    (gnus-html-image-url-blocked-p url blocked-images))
+		(make-text-button start end
+				  'help-echo url
+				  'keymap gnus-html-image-map)
+	      ;; Non-blocked url
+	      (let ((width
+		     (when (string-match "width=\"?\\([0-9]+\\)" parameters)
+		       (string-to-number (match-string 1 parameters))))
+		    (height
+		     (when (string-match "height=\"?\\([0-9]+\\)" parameters)
+		       (string-to-number (match-string 1 parameters)))))
+		;; Don't fetch images that are really small.  They're
+		;; probably tracking pictures.
+		(when (and (or (null height)
+			       (> height 4))
+			   (or (null width)
+			       (> width 4)))
+		  (gnus-html-display-image url start end alt-text))))))))))
 
-(defun gnus-html-display-image (url start end &optional alt-text)
+(defun gnus-html-display-image (url _start _end &optional alt-text)
   "Display image at URL on text from START to END.
 Use ALT-TEXT for the image string."
   (or alt-text (setq alt-text "*"))
@@ -262,7 +248,7 @@ Use ALT-TEXT for the image string."
       (gnus-html-put-image (gnus-html-get-image-data url) url alt-text))))
 
 (defun gnus-html-wash-tags ()
-  (let (tag parameters string start end images url)
+  (let (tag parameters start end url) ;; string images
     (gnus-html-pre-wash)
     (gnus-html-wash-images)
 
@@ -343,10 +329,10 @@ Use ALT-TEXT for the image string."
       (replace-match "" t t))
     (mm-url-decode-entities)))
 
-(defun gnus-html-insert-image (&rest args)
+(defun gnus-html-insert-image (&rest _args)
   "Fetch and insert the image under point."
   (interactive)
-  (apply 'gnus-html-display-image (get-text-property (point) 'gnus-image)))
+  (apply #'gnus-html-display-image (get-text-property (point) 'gnus-image)))
 
 (defun gnus-html-show-alt-text ()
   "Show the ALT text of the image under point."
@@ -450,11 +436,9 @@ Return a string with image data."
                   (let ((image (gnus-rescale-image image (gnus-html-maximum-image-size))))
                     (delete-region start end)
                     (gnus-put-image image alt-text 'external)
-		    (widget-convert-button
-		     'url-link start (point)
-		     :help-echo alt-text
-		     :keymap gnus-html-displayed-image-map
-		     url)
+		    (make-text-button start (point)
+				      'help-echo alt-text
+				      'keymap gnus-html-displayed-image-map)
                     (put-text-property start (point) 'gnus-alt-text alt-text)
                     (when url
 		      (add-text-properties
