@@ -1,6 +1,6 @@
 ;;; minibuf-eldef.el --- Only show defaults in prompts when applicable  -*- lexical-binding: t -*-
 ;;
-;; Copyright (C) 2000-2017 Free Software Foundation, Inc.
+;; Copyright (C) 2000-2022 Free Software Foundation, Inc.
 ;;
 ;; Author: Miles Bader <miles@gnu.org>
 ;; Keywords: convenience
@@ -36,10 +36,24 @@
 (defvar minibuffer-eldef-shorten-default)
 
 (defun minibuffer-default--in-prompt-regexps ()
-  `(("\\( (default\\(?: is\\)? \\(.*\\))\\):? \\'"
-     1 ,(if minibuffer-eldef-shorten-default " [\\2]"))
-    ("([^(]+?\\(, default\\(?: is\\)? \\(.*\\)\\)):? \\'" 1)
-    ("\\( \\[.*\\]\\):? *\\'" 1)))
+  (cons
+   (list
+    (concat
+     "\\("
+     (if (string-match "%s" minibuffer-default-prompt-format)
+         (concat
+          (regexp-quote (substring minibuffer-default-prompt-format
+                                   0 (match-beginning 0)))
+          "\\(.*?\\)"
+          (regexp-quote (substring minibuffer-default-prompt-format
+                                   (match-end 0))))
+       (regexp-quote minibuffer-default-prompt-format))
+     "\\): ")
+    1 (and minibuffer-eldef-shorten-default " [\\2]"))
+   `(("\\( (default\\(?: is\\)? \\(.*\\))\\):? \\'"
+      1 ,(if minibuffer-eldef-shorten-default " [\\2]"))
+     ("([^(]+?\\(, default\\(?: is\\)? \\(.*\\)\\)):? \\'" 1)
+     ("\\( \\[.*\\]\\):? *\\'" 1))))
 
 (defcustom minibuffer-eldef-shorten-default nil
   "If non-nil, shorten \"(default ...)\" to \"[...]\" in minibuffer prompts."
@@ -146,7 +160,7 @@ The prompt and initial input should already have been inserted."
 ;; post-command-hook to swap prompts when necessary
 (defun minibuf-eldef-update-minibuffer ()
   "Update a minibuffer's prompt to include a default only when applicable.
-This is intended to be used as a minibuffer post-command-hook for
+This is intended to be used as a minibuffer `post-command-hook' for
 `minibuffer-electric-default-mode'; the minibuffer should have already
 been set up by `minibuf-eldef-setup-minibuffer'."
   (unless (eq minibuf-eldef-showing-default-in-prompt
@@ -163,9 +177,6 @@ been set up by `minibuf-eldef-setup-minibuffer'."
 ;;;###autoload
 (define-minor-mode minibuffer-electric-default-mode
   "Toggle Minibuffer Electric Default mode.
-With a prefix argument ARG, enable Minibuffer Electric Default
-mode if ARG is positive, and disable it otherwise.  If called
-from Lisp, enable the mode if ARG is omitted or nil.
 
 Minibuffer Electric Default mode is a global minor mode.  When
 enabled, minibuffer prompts that show a default value only show
