@@ -1,6 +1,6 @@
-;;; pong.el --- classical implementation of pong
+;;; pong.el --- classical implementation of pong  -*- lexical-binding:t -*-
 
-;; Copyright (C) 1999-2018 Free Software Foundation, Inc.
+;; Copyright (C) 1999-2022 Free Software Foundation, Inc.
 
 ;; Author: Benjamin Drieu <bdrieu@april.org>
 ;; Keywords: games
@@ -33,88 +33,72 @@
 ;;; Customization
 
 (defgroup pong nil
-  "Emacs-Lisp implementation of the classical game pong."
+  "Emacs Lisp implementation of the classical game pong."
   :tag "Pong"
   :group 'games)
 
 (defcustom pong-buffer-name "*Pong*"
   "Name of the buffer used to play."
-  :group 'pong
   :type '(string))
 
 (defcustom pong-width 50
   "Width of the playfield."
-  :group 'pong
   :type '(integer))
 
 (defcustom pong-height (min 30 (- (frame-height) 6))
   "Height of the playfield."
-  :group 'pong
   :type '(integer))
 
 (defcustom pong-bat-width 3
   "Width of the bats for pong."
-  :group 'pong
   :type '(integer))
 
 (defcustom pong-blank-color "black"
   "Color used for background."
-  :group 'pong
   :type 'color)
 
 (defcustom pong-bat-color "yellow"
   "Color used for bats."
-  :group 'pong
   :type 'color)
 
 (defcustom pong-ball-color "red"
   "Color used for the ball."
-  :group 'pong
   :type 'color)
 
 (defcustom pong-border-color "white"
   "Color used for pong borders."
-  :group 'pong
   :type 'color)
 
 (defcustom pong-left-key "4"
   "Alternate key to press for bat 1 to go up (primary one is [left])."
-  :group 'pong
   :type '(restricted-sexp :match-alternatives (stringp vectorp)))
 
 (defcustom pong-right-key "6"
   "Alternate key to press for bat 1 to go down (primary one is [right])."
-  :group 'pong
   :type '(restricted-sexp :match-alternatives (stringp vectorp)))
 
 (defcustom pong-up-key "8"
   "Alternate key to press for bat 2 to go up (primary one is [up])."
-  :group 'pong
   :type '(restricted-sexp :match-alternatives (stringp vectorp)))
 
 (defcustom pong-down-key "2"
   "Alternate key to press for bat 2 to go down (primary one is [down])."
-  :group 'pong
   :type '(restricted-sexp :match-alternatives (stringp vectorp)))
 
 (defcustom pong-quit-key "q"
   "Key to press to quit pong."
-  :group 'pong
   :type '(restricted-sexp :match-alternatives (stringp vectorp)))
 
 (defcustom pong-pause-key "p"
   "Key to press to pause pong."
-  :group 'pong
   :type '(restricted-sexp :match-alternatives (stringp vectorp)))
 
 (defcustom pong-resume-key "p"
   "Key to press to resume pong."
-  :group 'pong
   :type '(restricted-sexp :match-alternatives (stringp vectorp)))
 
 (defcustom pong-timer-delay 0.1
   "Time to wait between every cycle."
-  :group 'pong
   :type 'number)
 
 
@@ -232,7 +216,6 @@
 
 (defun pong-init-buffer ()
   "Initialize pong buffer and draw stuff thanks to gamegrid library."
-  (interactive)
   (get-buffer-create pong-buffer-name)
   (switch-to-buffer pong-buffer-name)
   (use-local-map pong-mode-map)
@@ -262,10 +245,10 @@
 
 
 (defun pong-move-left ()
-  "Move bat 2 up.
+  "Move bat 1 up.
 This is called left for historical reasons, since in some pong
 implementations you move with left/right paddle."
-  (interactive)
+  (interactive nil pong-mode)
   (if (> pong-bat-player1 1)
       (and
        (setq pong-bat-player1 (1- pong-bat-player1))
@@ -274,8 +257,8 @@ implementations you move with left/right paddle."
 
 
 (defun pong-move-right ()
-  "Move bat 2 up."
-  (interactive)
+  "Move bat 1 down."
+  (interactive nil pong-mode)
   (if (< (+ pong-bat-player1 pong-bat-width) (1- pong-height))
       (and
        (setq pong-bat-player1 (1+ pong-bat-player1))
@@ -285,7 +268,7 @@ implementations you move with left/right paddle."
 
 (defun pong-move-up ()
   "Move bat 2 up."
-  (interactive)
+  (interactive nil pong-mode)
   (if (> pong-bat-player2 1)
       (and
        (setq pong-bat-player2 (1- pong-bat-player2))
@@ -295,7 +278,7 @@ implementations you move with left/right paddle."
 
 (defun pong-move-down ()
   "Move bat 2 down."
-  (interactive)
+  (interactive nil pong-mode)
   (if (< (+ pong-bat-player2 pong-bat-width) (1- pong-height))
       (and
        (setq pong-bat-player2 (1+ pong-bat-player2))
@@ -349,46 +332,61 @@ detection and checks if a player scores."
 
     (let ((old-x pong-x)
 	  (old-y pong-y))
+      ;; Erase the last ball position.
+      (when (and (> old-y 0)
+                 (< old-y (- pong-height 1)))
+        ;; If the ball hit the bat in the column where bats are positioned,
+        ;; and therefore changed its x direction, draw again the bat cell.
+        (if (or (and (= old-x 2) (< 0 pong-xx))
+                (and (= old-x (- pong-width 3)) (> 0 pong-xx)))
+            (gamegrid-set-cell old-x old-y pong-bat)
+          (gamegrid-set-cell old-x old-y pong-blank)))
 
+      ;; Update the ball position.
       (setq pong-x (+ pong-x pong-xx))
-      (setq pong-y (+ pong-y pong-yy))
-
-      (if (and (> old-y 0)
-	       (< old-y (- pong-height 1)))
-	  (gamegrid-set-cell old-x old-y pong-blank))
+      ;; If the ball would go out of bounds, put it against the border.
+      (cond
+       ((<= (+ pong-y pong-yy) 0)
+        (setq pong-yy (- pong-yy))
+        (setq pong-y 1))
+       ((>= (+ pong-y pong-yy) (- pong-height 1))
+        (setq pong-yy (- pong-yy))
+        (setq pong-y (- pong-height 2)))
+       (t
+        (setq pong-y (+ pong-y pong-yy))
+        ;; Check if the ball is against the border now,
+        ;; and change the y direction if it is.
+        (when (or (<= pong-y 1) (>= pong-y (- pong-height 2)))
+          (setq pong-yy (- pong-yy)))))
 
+      ;; Draw the ball in its new position.
       (if (and (> pong-y 0)
 	       (< pong-y (- pong-height 1)))
 	  (gamegrid-set-cell pong-x pong-y pong-ball))
 
+      ;; Hit bat, score a goal, or nothing.
       (cond
-       ((or (= pong-x 3) (= pong-x 2))
-	(if (and (>= pong-y pong-bat-player1)
-		 (< pong-y (+ pong-bat-player1 pong-bat-width)))
-	    (and
-	     (setq pong-yy (+ pong-yy
-			      (cond
-			       ((= pong-y pong-bat-player1) -1)
-			       ((= pong-y (1+ pong-bat-player1)) 0)
-			       (t 1))))
-	     (setq pong-xx (- pong-xx)))))
+       ((and (or (= pong-x 3) (= pong-x 2))
+             (> 0 pong-xx) ; Collide with the bat if headed towards it.
+             (>= pong-y pong-bat-player1)
+             (< pong-y (+ pong-bat-player1 pong-bat-width)))
+        (setq pong-yy (+ pong-yy
+			 (cond
+			  ((= pong-y pong-bat-player1) -1)
+			  ((= pong-y (1+ pong-bat-player1)) 0)
+			  (t 1))))
+        (setq pong-xx (- pong-xx)))
 
-       ((or (= pong-x (- pong-width 4)) (= pong-x (- pong-width 3)))
-	(if (and (>= pong-y pong-bat-player2)
-		 (< pong-y (+ pong-bat-player2 pong-bat-width)))
-	    (and
-	     (setq pong-yy (+ pong-yy
-			      (cond
-			       ((= pong-y pong-bat-player2) -1)
-			       ((= pong-y (1+ pong-bat-player2)) 0)
-			       (t 1))))
-	     (setq pong-xx (- pong-xx)))))
-
-       ((<= pong-y 1)
-	(setq pong-yy (- pong-yy)))
-
-       ((>= pong-y (- pong-height 2))
-	(setq pong-yy (- pong-yy)))
+       ((and (or (= pong-x (- pong-width 4)) (= pong-x (- pong-width 3)))
+             (< 0 pong-xx) ; Collide with the bat if headed towards it.
+             (>= pong-y pong-bat-player2)
+             (< pong-y (+ pong-bat-player2 pong-bat-width)))
+	(setq pong-yy (+ pong-yy
+			 (cond
+			  ((= pong-y pong-bat-player2) -1)
+			  ((= pong-y (1+ pong-bat-player2)) 0)
+			  (t 1))))
+	(setq pong-xx (- pong-xx)))
 
        ((< pong-x 1)
 	(setq pong-score-player2 (1+ pong-score-player2))
@@ -413,7 +411,7 @@ detection and checks if a player scores."
 
 (defun pong-pause ()
   "Pause the game."
-  (interactive)
+  (interactive nil pong-mode)
   (gamegrid-kill-timer)
   ;; Oooohhh ugly.  I don't know why, gamegrid-kill-timer don't do the
   ;; jobs it is made for.  So I have to do it "by hand".  Anyway, next
@@ -425,7 +423,7 @@ detection and checks if a player scores."
 
 (defun pong-resume ()
   "Resume a paused game."
-  (interactive)
+  (interactive nil pong-mode)
   (define-key pong-mode-map pong-pause-key 'pong-pause)
   (gamegrid-start-timer pong-timer-delay 'pong-update-game))
 
@@ -433,7 +431,7 @@ detection and checks if a player scores."
 
 (defun pong-quit ()
   "Quit the game and kill the pong buffer."
-  (interactive)
+  (interactive nil pong-mode)
   (gamegrid-kill-timer)
   ;; Be sure not to draw things in another buffer and wait for some
   ;; time.
