@@ -1,5 +1,5 @@
 /* Interface to Little CMS
-   Copyright (C) 2017-2021 Free Software Foundation, Inc.
+   Copyright (C) 2017-2022 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -254,8 +254,7 @@ parse_viewing_conditions (Lisp_Object view, const cmsCIEXYZ *wp,
 #define PARSE_VIEW_CONDITION_INT(field)					\
   if (CONSP (view) && FIXNATP (XCAR (view)))				\
     {									\
-      CHECK_RANGED_INTEGER (XCAR (view), 1, 4);				\
-      vc->field = XFIXNUM (XCAR (view));					\
+      vc->field = check_integer_range (XCAR (view), 1, 4);		\
       view = XCDR (view);						\
     }									\
   else									\
@@ -317,7 +316,7 @@ jab_to_jch (const lcmsJab_t *jab, cmsJCh *jch, double FL, double c1, double c2)
 }
 
 DEFUN ("lcms-xyz->jch", Flcms_xyz_to_jch, Slcms_xyz_to_jch, 1, 3, 0,
-       doc: /* Convert CIE CAM02 JCh to CIE XYZ.
+       doc: /* Convert CIE XYZ to CIE CAM02 JCh.
 COLOR is a list (X Y Z), with Y scaled about unity.
 Optional arguments WHITEPOINT and VIEW are the same as in `lcms-cam02-ucs',
 which see.  */)
@@ -353,7 +352,7 @@ which see.  */)
 }
 
 DEFUN ("lcms-jch->xyz", Flcms_jch_to_xyz, Slcms_jch_to_xyz, 1, 3, 0,
-       doc: /* Convert CIE XYZ to CIE CAM02 JCh.
+       doc: /* Convert CIE CAM02 JCh to CIE XYZ.
 COLOR is a list (J C h), where lightness of white is equal to 100, and hue
 is given in degrees.
 Optional arguments WHITEPOINT and VIEW are the same as in `lcms-cam02-ucs',
