@@ -1,6 +1,6 @@
 /* Font back-end driver for the NeXT/Open/GNUstep and macOS window system.
    See font.h
-   Copyright (C) 2006-2021 Free Software Foundation, Inc.
+   Copyright (C) 2006-2022 Free Software Foundation, Inc.
 
 This file is part of GNU Emacs.
 
@@ -39,9 +39,7 @@ Author: Adrian Robert (arobert@cogsci.ucsd.edu)
 #include "pdumper.h"
 
 /* TODO: Drop once we can assume gnustep-gui 0.17.1.  */
-#ifdef NS_IMPL_GNUSTEP
 #import <AppKit/NSFontDescriptor.h>
-#endif
 
 #define NSFONT_TRACE 0
 #define LCD_SMOOTHING_MARGIN 2
@@ -237,12 +235,6 @@ ns_char_width (NSFont *sfont, int c)
   CGFloat w = -1.0;
   NSString *cstr = [NSString stringWithFormat: @"%c", c];
 
-#ifdef NS_IMPL_COCOA
-  NSGlyph glyph = [sfont glyphWithName: cstr];
-  if (glyph)
-    w = [sfont advancementForGlyph: glyph].width;
-#endif
-
   if (w < 0.0)
     {
       NSDictionary *attrsDictionary =
@@ -273,12 +265,6 @@ ns_ascii_average_width (NSFont *sfont)
       ascii_printable = [[NSString alloc] initWithFormat: @"%s", chars];
     }
 
-#ifdef NS_IMPL_COCOA
-  NSGlyph glyph = [sfont glyphWithName: ascii_printable];
-  if (glyph)
-    w = [sfont advancementForGlyph: glyph].width;
-#endif
-
   if (w < (CGFloat) 0.0)
     {
       NSDictionary *attrsDictionary =
@@ -291,30 +277,36 @@ ns_ascii_average_width (NSFont *sfont)
 
 
 /* Return whether set1 covers set2 to a reasonable extent given by pct.
-   We check, out of each 16 Unicode char range containing chars in set2,
-   whether at least one character is present in set1.
-   This must be true for pct of the pairs to consider it covering.  */
+
+   The GNUstep bitmap representation doesn't match Apple's
+   description.  It appears to be a single block of bytes, not broken
+   up into planes, where the last byte contains the highest character
+   the character set supports.  */
 static BOOL
 ns_charset_covers(NSCharacterSet *set1, NSCharacterSet *set2, float pct)
 {
-    const unsigned short *bytes1 = [[set1 bitmapRepresentation] bytes];
-    const unsigned short *bytes2 = [[set2 bitmapRepresentation] bytes];
-    int i, off = 0, tot = 0;
+  NSData *font = [set1 bitmapRepresentation];
+  NSData *script = [set2 bitmapRepresentation];
 
-    /* Work around what appears to be a GNUstep bug.
-       See <https://bugs.gnu.org/11853>.  */
-    if (! (bytes1 && bytes2))
-      return NO;
+  uint8_t *fontPlane = (uint8_t *)[font bytes];
+  uint8_t *scriptPlane = (uint8_t *)[script bytes];
 
-    for (i=0; i<4096; i++, bytes1++, bytes2++)
-	if (*bytes2)
-	  {
-	    tot++;
-	    if (*bytes1 == 0)  // *bytes1 & *bytes2 != *bytes2
-		off++;
-	  }
-    // fprintf(stderr, "off = %d\ttot = %d\n", off,tot);
-    return (float)off / tot < 1.0F - pct;
+  int covered = 0, total = 0;
+
+  for (ptrdiff_t b = 0 ; b < [script length] ; b++)
+    for (int i = 0 ; i < 8 ; i++)
+      {
+        if (*(scriptPlane + b) & (1 << i))
+          {
+            total++;
+
+            if (b < [font length]
+                && *(fontPlane + b) & (1 << i))
+              covered++;
+          }
+      }
+
+  return (float)covered / total >= 1.0F - pct;
 }
 
 
@@ -343,7 +335,7 @@ static NSString
 {
     Lisp_Object script = assq_no_quit (XCAR (otf), Votf_script_alist);
     return CONSP (script)
-	? [NSString stringWithUTF8String: SSDATA (SYMBOL_NAME (XCDR ((script))))]
+	? [NSString stringWithLispString: SYMBOL_NAME (XCDR ((script)))]
 	: @"";
 }
 
@@ -359,7 +351,7 @@ static NSString
         if (!strncmp (SSDATA (r), reg, SBYTES (r)))
           {
             script = XCDR (XCAR (rts));
-            return [NSString stringWithUTF8String: SSDATA (SYMBOL_NAME (script))];
+            return [NSString stringWithLispString: SYMBOL_NAME (script)];
           }
         rts = XCDR (rts);
       }
@@ -384,8 +376,7 @@ static NSString
 	  {
 	    Lisp_Object key = XCAR (tmp), val = XCDR (tmp);
 	    if (EQ (key, QCscript) && SYMBOLP (val))
-		return [NSString stringWithUTF8String:
-		            SSDATA (SYMBOL_NAME (val))];
+		return [NSString stringWithLispString: SYMBOL_NAME (val)];
 	    if (EQ (key, QClang) && SYMBOLP (val))
 		return ns_lang_to_script (val);
 	    if (EQ (key, QCotf) && CONSP (val) && SYMBOLP (XCAR (val)))
@@ -511,10 +502,6 @@ static NSSet
 	      }
             [charset release];
 	  }
-#ifdef NS_IMPL_COCOA
-	if ([families count] == 0)
-	    [families addObject: @"LastResort"];
-#endif
 	[scriptToFamilies setObject: families forKey: script];
       }
 
@@ -719,7 +706,7 @@ nsfont_open (struct frame *f, Lisp_Object font_entity, int pixel_size)
      when setting family in ns_spec_to_descriptor().  */
   if (ns_attribute_fvalue (fontDesc, NSFontWeightTrait) > 0.50F)
       traits |= NSBoldFontMask;
-  if (fabs (ns_attribute_fvalue (fontDesc, NSFontSlantTrait) > 0.05F))
+  if (ns_attribute_fvalue (fontDesc, NSFontSlantTrait) > 0.05F)
       traits |= NSItalicFontMask;
 
   /* see https://web.archive.org/web/20100201175731/http://cocoadev.com/forums/comments.php?DiscussionID=74 */
@@ -734,11 +721,6 @@ nsfont_open (struct frame *f, Lisp_Object font_entity, int pixel_size)
                                 traits: traits & ~NSItalicFontMask
                                 weight: fixLeopardBug size: pixel_size];
     }
-#ifdef NS_IMPL_COCOA
-  /* LastResort not really a family */
-  if (nsfont == nil && [@"LastResort" isEqualToString: family])
-      nsfont = [NSFont fontWithName: @"LastResort" size: pixel_size];
-#endif
 
   if (nsfont == nil)
     {
@@ -765,12 +747,7 @@ nsfont_open (struct frame *f, Lisp_Object font_entity, int pixel_size)
   font_info->metrics = xzalloc (0x100 * sizeof *font_info->metrics);
 
   /* for metrics */
-#ifdef NS_IMPL_COCOA
-  sfont = [nsfont screenFontWithRenderingMode:
-                    NSFontAntialiasedIntegerAdvancementsRenderingMode];
-#else
   sfont = [nsfont screenFont];
-#endif
 
   if (sfont == nil)
     sfont = nsfont;
@@ -797,11 +774,7 @@ nsfont_open (struct frame *f, Lisp_Object font_entity, int pixel_size)
      * intended.  */
     CGFloat adjusted_descender = [sfont descender] + 0.0001;
 
-#ifdef NS_IMPL_GNUSTEP
     font_info->nsfont = sfont;
-#else
-    font_info->nsfont = nsfont;
-#endif
     [font_info->nsfont retain];
 
     /* set up ns_font (defined in nsgui.h) */
@@ -834,32 +807,6 @@ nsfont_open (struct frame *f, Lisp_Object font_entity, int pixel_size)
     font_info->max_bounds.rbearing =
       lrint (brect.size.width - (CGFloat) font_info->width);
 
-#ifdef NS_IMPL_COCOA
-    /* set up synthItal and the CG font */
-    font_info->synthItal = synthItal;
-    {
-      ATSFontRef atsFont = ATSFontFindFromPostScriptName
-        ((CFStringRef)[nsfont fontName], kATSOptionFlagsDefault);
-
-      if (atsFont == kATSFontRefUnspecified)
-        {
-          /* see if we can get it by dropping italic (then synthesizing) */
-          atsFont = ATSFontFindFromPostScriptName ((CFStringRef)
-              [[fontMgr convertFont: nsfont toNotHaveTrait: NSItalicFontMask]
-                fontName], kATSOptionFlagsDefault);
-          if (atsFont != kATSFontRefUnspecified)
-              font_info->synthItal = YES;
-          else
-            {
-              /* last resort fallback */
-              atsFont = ATSFontFindFromPostScriptName
-                ((CFStringRef)@"Monaco", kATSOptionFlagsDefault);
-            }
-        }
-      font_info->cgfont = CGFontCreateWithPlatformFont ((void *) &atsFont);
-    }
-#endif
-
     /* set up metrics portion of font struct */
     font->ascent = lrint([sfont ascender]);
     font->descent = -lrint(floor(adjusted_descender));
@@ -901,9 +848,6 @@ nsfont_close (struct font *font)
       xfree (font_info->glyphs);
       xfree (font_info->metrics);
       [font_info->nsfont release];
-#ifdef NS_IMPL_COCOA
-      CGFontRelease (font_info->cgfont);
-#endif
       xfree (font_info->name);
       font_info->name = NULL;
     }
@@ -994,17 +938,12 @@ nsfont_draw (struct glyph_string *s, int from, int to, int x, int y,
 {
   static unsigned char cbuf[1024];
   unsigned char *c = cbuf;
-#ifdef NS_IMPL_GNUSTEP
 #if GNUSTEP_GUI_MAJOR_VERSION > 0 || GNUSTEP_GUI_MINOR_VERSION > 22
   static CGFloat advances[1024];
   CGFloat *adv = advances;
 #else
   static float advances[1024];
   float *adv = advances;
-#endif
-#else
-  static CGSize advances[1024];
-  CGSize *adv = advances;
 #endif
   struct face *face;
   NSRect r;
@@ -1043,7 +982,7 @@ nsfont_draw (struct glyph_string *s, int from, int to, int x, int y,
 
   r.origin.x = s->x;
   if (s->face->box != FACE_NO_BOX && s->first_glyph->left_box_line_p)
-    r.origin.x += abs (s->face->box_line_width);
+    r.origin.x += max (s->face->box_vertical_line_width, 0);
 
   r.origin.y = s->y;
   r.size.height = FONT_HEIGHT (font);
@@ -1073,11 +1012,7 @@ nsfont_draw (struct glyph_string *s, int from, int to, int x, int y,
 		else
 		  {
 		    cwidth = LGLYPH_WADJUST (glyph);
-#ifdef NS_IMPL_GNUSTEP
 		    *(adv-1) += LGLYPH_XOFF (glyph);
-#else
-		    (*(adv-1)).width += LGLYPH_XOFF (glyph);
-#endif
 		  }
 	      }
           }
@@ -1088,12 +1023,8 @@ nsfont_draw (struct glyph_string *s, int from, int to, int x, int y,
             cwidth = font->metrics[hi][lo].width;
           }
         twidth += cwidth;
-#ifdef NS_IMPL_GNUSTEP
         *adv++ = cwidth;
-        CHAR_STRING_ADVANCE (*t, c); /* This converts the char to UTF-8.  */
-#else
-        (*adv++).width = cwidth;
-#endif
+        c += CHAR_STRING (*t, c); /* This converts the char to UTF-8.  */
       }
     len = adv - advances;
     r.size.width = twidth;
@@ -1105,7 +1036,7 @@ nsfont_draw (struct glyph_string *s, int from, int to, int x, int y,
     {
       NSRect br = r;
       int fibw = FRAME_INTERNAL_BORDER_WIDTH (s->f);
-      int mbox_line_width = max (s->face->box_line_width, 0);
+      int mbox_line_width = max (s->face->box_vertical_line_width, 0);
 
       if (s->row->full_width_p)
         {
@@ -1129,9 +1060,10 @@ nsfont_draw (struct glyph_string *s, int from, int to, int x, int y,
         }
       else
         {
-          int correction = abs (s->face->box_line_width)+1;
+          int correction = abs (s->face->box_horizontal_line_width)+1;
           br.origin.y += correction;
           br.size.height -= 2*correction;
+          correction = abs (s->face->box_vertical_line_width)+1;
           br.origin.x += correction;
           br.size.width -= 2*correction;
         }
@@ -1162,7 +1094,6 @@ nsfont_draw (struct glyph_string *s, int from, int to, int x, int y,
               : FRAME_BACKGROUND_COLOR (s->f)));
 
   /* render under GNUstep using DPS */
-#ifdef NS_IMPL_GNUSTEP
   {
     NSGraphicsContext *context = GSCurrentContext ();
 
@@ -1191,61 +1122,6 @@ nsfont_draw (struct glyph_string *s, int from, int to, int x, int y,
     DPSgrestore (context);
   }
 
-#else  /* NS_IMPL_COCOA */
-  {
-    CGContextRef gcontext =
-      [[NSGraphicsContext currentContext] graphicsPort];
-    static CGAffineTransform fliptf;
-    static BOOL firstTime = YES;
-
-    if (firstTime)
-      {
-        firstTime = NO;
-        fliptf = CGAffineTransformMakeScale (1.0, -1.0);
-      }
-
-    CGContextSaveGState (gcontext);
-
-    // Used to be Fix2X (kATSItalicQDSkew), but Fix2X is deprecated
-    // and kATSItalicQDSkew is 0.25.
-    fliptf.c =  font->synthItal ? 0.25 : 0.0;
-
-    CGContextSetFont (gcontext, font->cgfont);
-    CGContextSetFontSize (gcontext, font->size);
-    if (NILP (ns_antialias_text) || font->size <= ns_antialias_threshold)
-      CGContextSetShouldAntialias (gcontext, 0);
-    else
-      CGContextSetShouldAntialias (gcontext, 1);
-
-    CGContextSetTextMatrix (gcontext, fliptf);
-
-    if (bgCol != nil)
-      {
-        /* foreground drawing; erase first to avoid overstrike */
-        [bgCol set];
-        CGContextSetTextDrawingMode (gcontext, kCGTextFillStroke);
-        CGContextSetTextPosition (gcontext, r.origin.x, r.origin.y);
-        CGContextShowGlyphsWithAdvances (gcontext, s->char2b, advances, len);
-        CGContextSetTextDrawingMode (gcontext, kCGTextFill);
-      }
-
-    [col set];
-
-    CGContextSetTextPosition (gcontext, r.origin.x, r.origin.y);
-    CGContextShowGlyphsWithAdvances (gcontext, s->char2b + from,
-                                     advances, len);
-
-    if (face->overstrike)
-      {
-        CGContextSetTextPosition (gcontext, r.origin.x+0.5, r.origin.y);
-        CGContextShowGlyphsWithAdvances (gcontext, s->char2b + from,
-                                         advances, len);
-      }
-
-    CGContextRestoreGState (gcontext);
-  }
-#endif  /* NS_IMPL_COCOA */
-
   unblock_input ();
   return to-from;
 }
@@ -1263,10 +1139,6 @@ nsfont_draw (struct glyph_string *s, int from, int to, int x, int y,
 static void
 ns_uni_to_glyphs (struct nsfont_info *font_info, unsigned char block)
 {
-#ifdef NS_IMPL_COCOA
-  static EmacsGlyphStorage *glyphStorage;
-  static char firstTime = 1;
-#endif
   unichar *unichars = xmalloc (0x101 * sizeof (unichar));
   unsigned int i, g, idx;
   unsigned short *glyphs;
@@ -1277,14 +1149,6 @@ ns_uni_to_glyphs (struct nsfont_info *font_info, unsigned char block)
 
   block_input ();
 
-#ifdef NS_IMPL_COCOA
-  if (firstTime)
-    {
-      firstTime = 0;
-      glyphStorage = [[EmacsGlyphStorage alloc] initWithCapacity: 0x100];
-    }
-#endif
-
   font_info->glyphs[block] = xmalloc (0x100 * sizeof (unsigned short));
   if (!unichars || !(font_info->glyphs[block]))
     emacs_abort ();
@@ -1298,38 +1162,12 @@ ns_uni_to_glyphs (struct nsfont_info *font_info, unsigned char block)
   unichars[0x100] = 0;
 
   {
-#ifdef NS_IMPL_COCOA
-    NSString *allChars = [[NSString alloc]
-                               initWithCharactersNoCopy: unichars
-                                                 length: 0x100
-                                           freeWhenDone: NO];
-    NSGlyphGenerator *glyphGenerator = [NSGlyphGenerator sharedGlyphGenerator];
-    /* NSCharacterSet *coveredChars = [nsfont coveredCharacterSet]; */
-    unsigned int numGlyphs = [font_info->nsfont numberOfGlyphs];
-    NSUInteger gInd = 0, cInd = 0;
-
-    [glyphStorage setString: allChars font: font_info->nsfont];
-    [glyphGenerator generateGlyphsForGlyphStorage: glyphStorage
-                        desiredNumberOfCharacters: glyphStorage->maxChar
-                                       glyphIndex: &gInd characterIndex: &cInd];
-#endif
     glyphs = font_info->glyphs[block];
     for (i = 0; i < 0x100; i++, glyphs++)
       {
-#ifdef NS_IMPL_GNUSTEP
         g = unichars[i];
-#else
-        g = glyphStorage->cglyphs[i];
-        /* TODO: is this a good check?  Maybe need to use coveredChars.  */
-        if (g > numGlyphs || g == NSNullGlyph)
-          g = INVALID_GLYPH; /* Hopefully unused...  */
-#endif
         *glyphs = g;
       }
-
-#ifdef NS_IMPL_COCOA
-    [allChars release];
-#endif
   }
 
   unblock_input ();
@@ -1351,19 +1189,12 @@ ns_glyph_metrics (struct nsfont_info *font_info, unsigned char block)
     fprintf (stderr, "%p\tComputing metrics for glyphs in block %d\n",
             font_info, block);
 
-#ifdef NS_IMPL_GNUSTEP
   /* not implemented yet (as of startup 0.18), so punt */
   if (numGlyphs == 0)
     numGlyphs = 0x10000;
-#endif
 
   block_input ();
-#ifdef NS_IMPL_COCOA
-  sfont = [font_info->nsfont screenFontWithRenderingMode:
-                      NSFontAntialiasedIntegerAdvancementsRenderingMode];
-#else
   sfont = [font_info->nsfont screenFont];
-#endif
 
   font_info->metrics[block] = xzalloc (0x100 * sizeof (struct font_metrics));
   if (!(font_info->metrics[block]))
@@ -1396,76 +1227,6 @@ ns_glyph_metrics (struct nsfont_info *font_info, unsigned char block)
 }
 
 
-#ifdef NS_IMPL_COCOA
-/* Helper for font glyph setup.  */
-@implementation EmacsGlyphStorage
-
-- init
-{
-  return [self initWithCapacity: 1024];
-}
-
-- initWithCapacity: (unsigned long) c
-{
-  self = [super init];
-  maxChar = 0;
-  maxGlyph = 0;
-  dict = [NSMutableDictionary new];
-  cglyphs = xmalloc (c * sizeof (CGGlyph));
-  return self;
-}
-
-- (void) dealloc
-{
-  if (attrStr != nil)
-    [attrStr release];
-  [dict release];
-  xfree (cglyphs);
-  [super dealloc];
-}
-
-- (void) setString: (NSString *)str font: (NSFont *)font
-{
-  [dict setObject: font forKey: NSFontAttributeName];
-  if (attrStr != nil)
-    [attrStr release];
-  attrStr = [[NSAttributedString alloc] initWithString: str attributes: dict];
-  maxChar = [str length];
-  maxGlyph = 0;
-}
-
-/* NSGlyphStorage protocol */
-- (NSUInteger)layoutOptions
-{
-  return 0;
-}
-
-- (NSAttributedString *)attributedString
-{
-  return attrStr;
-}
-
-- (void)insertGlyphs: (const NSGlyph *)glyphs length: (NSUInteger)length
-        forStartingGlyphAtIndex: (NSUInteger)glyphIndex
-        characterIndex: (NSUInteger)charIndex
-{
-  len = glyphIndex+length;
-  for (i =glyphIndex; i<len; i++)
-    cglyphs[i] = glyphs[i-glyphIndex];
-  if (len > maxGlyph)
-    maxGlyph = len;
-}
-
-- (void)setIntAttribute: (NSInteger)attributeTag value: (NSInteger)val
-        forGlyphAtIndex: (NSUInteger)glyphIndex
-{
-  return;
-}
-
-@end
-#endif /* NS_IMPL_COCOA */
-
-
 /* Debugging */
 void
 ns_dump_glyphstring (struct glyph_string *s)
