@@ -1,6 +1,6 @@
 ;;; map-tests.el --- Tests for map.el  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2015-2021 Free Software Foundation, Inc.
+;; Copyright (C) 2015-2022 Free Software Foundation, Inc.
 
 ;; Author: Nicolas Petton <nicolas@petton.fr>
 ;; Maintainer: emacs-devel@gnu.org
@@ -422,6 +422,9 @@ Evaluate BODY for each created map."
     (should (map-every-p (lambda (k _v) (zerop k)) map))))
 
 (ert-deftest test-map-into ()
+  ;; This test is unstable in Emacs 28, but the problem has been fixed
+  ;; in Emacs 29 (bug#46722).
+  :tags '(:unstable)
   (let* ((plist '(a 1 b 2))
          (alist '((a . 1) (b . 2)))
          (ht (map-into alist 'hash-table))
@@ -440,6 +443,9 @@ Evaluate BODY for each created map."
 
 (ert-deftest test-map-into-hash-test ()
   "Test `map-into' with different hash-table test functions."
+  ;; This test is unstable in Emacs 28, but the problem has been fixed
+  ;; in Emacs 29 (bug#46722).
+  :tags '(:unstable)
   (should (eq (hash-table-test (map-into () 'hash-table)) #'equal))
   (should (eq (hash-table-test (map-into () '(hash-table))) #'eql))
   (should (eq (hash-table-test (map-into () '(hash-table :test eq))) #'eq))
@@ -470,6 +476,9 @@ Evaluate BODY for each created map."
 
 (ert-deftest test-map-merge ()
   "Test `map-merge'."
+  ;; This test is unstable in Emacs 28, but the problem has been fixed
+  ;; in Emacs 29 (bug#46722).
+  :tags '(:unstable)
   (should (equal (sort (map-merge 'list '(a 1) '((b . 2) (c . 3))
                                   #s(hash-table data (c 4)))
                        (lambda (x y) (string< (car x) (car y))))
@@ -479,6 +488,9 @@ Evaluate BODY for each created map."
   (should (equal (map-merge 'plist () '(:a 1)) '(:a 1))))
 
 (ert-deftest test-map-merge-with ()
+  ;; This test is unstable in Emacs 28, but the problem has been fixed
+  ;; in Emacs 29 (bug#46722).
+  :tags '(:unstable)
   (should (equal (sort (map-merge-with 'list #'+
                                        '((1 . 2))
                                        '((1 . 3) (2 . 4))
@@ -491,6 +503,9 @@ Evaluate BODY for each created map."
 
 (ert-deftest test-map-merge-empty ()
   "Test merging of empty maps."
+  ;; This test is unstable in Emacs 28, but the problem has been fixed
+  ;; in Emacs 29 (bug#46722).
+  :tags '(:unstable)
   (should-not (map-merge 'list))
   (should-not (map-merge 'alist))
   (should-not (map-merge 'plist))
