@@ -9,21 +9,21 @@
 (require 'macro-aux)
 
 (defmacro macro-problem-1 ( &rest forms)
-  "Description"
+  "Description."
   `(progn ,(cadr (car forms))))
 
 
 (defun macro-problem-func ()
-  ""
+  "Description."
   (list (macro-problem-1 '1 'b)
         (macro-aux-1 'a 'b)))
 
 (defmacro macro-problem-3 (&rest _)
-  "Description"
+  "Description."
   10)
 
 (defun macro-problem-10-and-90 ()
-  ""
+  "Description."
   (list (macro-problem-3 haha) (macro-aux-3 hehe)))
 
 (provide 'macro-problem)
