@@ -1,4 +1,4 @@
-;;; simple-single.el --- A single-file package with no dependencies
+;;; simple-single.el --- A single-file package with no dependencies  -*- lexical-binding:t -*-
 
 ;; Author: J. R. Hacker <jrh@example.com>
 ;; Version: 1.3
@@ -8,12 +8,12 @@
 ;;; Commentary:
 
 ;; This package provides a minor mode to frobnicate and/or bifurcate
-;; any flanges you desire. To activate it, type "C-M-r M-3 butterfly"
+;; any flanges you desire.  To activate it, type "C-M-r M-3 butterfly"
 ;; and all your dreams will come true.
 
 ;;; Code:
 
-(defgroup simple-single nil "Simply a file"
+(defgroup simple-single nil "Simply a file."
   :group 'lisp)
 
 (defcustom simple-single-super-sunday t
@@ -26,7 +26,7 @@
 
 ;;;###autoload
 (define-minor-mode simple-single-mode
-  "It does good things to stuff")
+  "It does good things to stuff.")
 
 (provide 'simple-single)
 
