@@ -1,6 +1,6 @@
 ;;; rx-tests.el --- tests for rx.el              -*- lexical-binding: t -*-
 
-;; Copyright (C) 2016-2021 Free Software Foundation, Inc.
+;; Copyright (C) 2016-2022 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -17,6 +17,8 @@
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
+;;; Code:
+
 (require 'ert)
 (require 'rx)
 
@@ -56,13 +58,17 @@
 (ert-deftest rx-def-in-or ()
   (rx-let ((a b)
            (b (or "abc" c))
-           (c ?a))
+           (c ?a)
+           (d (any "a-z")))
     (should (equal (rx (or a (| "ab" "abcde") "abcd"))
-                   "\\(?:a\\(?:b\\(?:c\\(?:de?\\)?\\)?\\)?\\)"))))
+                   "\\(?:a\\(?:b\\(?:c\\(?:de?\\)?\\)?\\)?\\)"))
+    (should (equal (rx (or ?m (not d)))
+                   "[^a-ln-z]"))))
 
 (ert-deftest rx-char-any ()
   "Test character alternatives with `]' and `-' (Bug#25123)."
   (should (equal
+           ;; relint suppression: Range .<-]. overlaps previous .]-{
            (rx string-start (1+ (char (?\] . ?\{) (?< . ?\]) (?- . ?:)))
                string-end)
            "\\`[.-:<-{-]+\\'")))
@@ -127,8 +133,12 @@
                  "[[:lower:][:upper:]-][^[:lower:][:upper:]-]"))
   (should (equal (rx (any "]" lower upper) (not (any "]" lower upper)))
                  "[][:lower:][:upper:]][^][:lower:][:upper:]]"))
-  (should (equal (rx (any "-a" "c-" "f-f" "--/*--"))
-                 "[*-/acf]"))
+  ;; relint suppression: Duplicated character .-.
+  ;; relint suppression: Single-character range .f-f
+  ;; relint suppression: Range .--/. overlaps previous .-
+  ;; relint suppression: Range .\*--. overlaps previous .--/
+  (should (equal (rx (any "-a" "c-" "f-f" "--/*--") (any "," "-" "A"))
+                 "[*-/acf][,A-]"))
   (should (equal (rx (any "]-a" ?-) (not (any "]-a" ?-)))
                  "[]-a-][^]-a-]"))
   (should (equal (rx (any "--]") (not (any "--]"))
@@ -140,6 +150,7 @@
                  "\\`a\\`[^z-a]"))
   (should (equal (rx (any "") (not (any "")))
                  "\\`a\\`[^z-a]"))
+  ;; relint suppression: Duplicated class .space.
   (should (equal (rx (any space ?a digit space))
                  "[a[:space:][:digit:]]"))
   (should (equal (rx (not "\n") (not ?\n) (not (any "\n")) (not-char ?\n)
@@ -147,6 +158,8 @@
           ".....")))
 
 (ert-deftest rx-pcase ()
+  (should (equal (pcase "i18n" ((rx (let x (+ digit))) (list 'ok x)))
+                 '(ok "18")))
   (should (equal (pcase "a 1 2 3 1 1 b"
                    ((rx (let u (+ digit)) space
                         (let v (+ digit)) space
@@ -155,6 +168,20 @@
                         (backref 1))
                     (list u v)))
                  '("1" "3")))
+  (should (equal (pcase "bz"
+                   ((rx "a" (let x nonl)) (list 1 x))
+                   (_ 'no))
+                 'no))
+  (should (equal (pcase "az"
+                   ((rx "a" (let x nonl)) (list 1 x))
+                   ((rx "b" (let x nonl)) (list 2 x))
+                   (_ 'no))
+                 '(1 "z")))
+  (should (equal (pcase "bz"
+                   ((rx "a" (let x nonl)) (list 1 x))
+                   ((rx "b" (let x nonl)) (list 2 x))
+                   (_ 'no))
+                 '(2 "z")))
   (let ((k "blue"))
     (should (equal (pcase "<blue>"
                      ((rx "<" (literal k) ">") 'ok))
@@ -162,7 +189,23 @@
   (should (equal (pcase "abc"
                    ((rx (? (let x alpha)) (?? (let y alnum)) ?c)
                     (list x y)))
-                 '("a" "b"))))
+                 '("a" "b")))
+  (should (equal (pcase 'not-a-string
+                   ((rx nonl) 'wrong)
+                   (_ 'correct))
+                 'correct))
+  (should (equal (pcase "PQR"
+                   ((and (rx (let a nonl)) (rx ?z))
+                    (list 'one a))
+                   ((rx (let b ?Q))
+                    (list 'two b)))
+                 '(two "Q")))
+  (should (equal (pcase-let (((rx ?B (let z nonl)) "ABC"))
+                   (list 'ok z))
+                 '(ok "C")))
+  (should (equal (pcase-let* (((rx ?E (let z nonl)) "DEF"))
+                   (list 'ok z))
+                 '(ok "F"))))
 
 (ert-deftest rx-kleene ()
   "Test greedy and non-greedy repetition operators."
@@ -379,6 +422,8 @@
 (ert-deftest rx-regexp ()
   (should (equal (rx (regexp "abc") (regex "[de]"))
                  "\\(?:abc\\)[de]"))
+  (should (equal (rx "a" (regexp "$"))
+                 "a\\(?:$\\)"))
   (let ((x "a*"))
     (should (equal (rx (regexp x) "b")
                    "\\(?:a*\\)b"))
@@ -392,6 +437,8 @@
                  "ab")))
 
 (ert-deftest rx-literal ()
+  (should (equal (rx (literal "$a"))
+                 "\\$a"))
   (should (equal (rx (literal (char-to-string 42)) nonl)
                  "\\*."))
   (let ((x "a+b"))
@@ -532,6 +579,11 @@
 
 (ert-deftest rx-compat ()
   "Test old symbol retained for compatibility (bug#37517)."
-  (should (equal (rx-submatch-n '(group-n 3 (+ nonl) eol)) "\\(?3:.+$\\)")))
+  (should (equal
+           (with-no-warnings
+             (rx-submatch-n '(group-n 3 (+ nonl) eol)))
+           "\\(?3:.+$\\)")))
 
 (provide 'rx-tests)
+
+;;; rx-tests.el ends here
